@@ -31,6 +31,15 @@ typedef struct {
                 return PyFloat_FromDouble(-Py_HUGE_VAL);        \
         } while(0)
 
+PyAPI_FUNC(int) PyFloat_Pack2(double x, char *p, int le);
+PyAPI_FUNC(int) PyFloat_Pack4(double x, char *p, int le);
+PyAPI_FUNC(int) PyFloat_Pack8(double x, char *p, int le);
+
+PyAPI_FUNC(double) PyFloat_Unpack2(const char *p, int le);
+PyAPI_FUNC(double) PyFloat_Unpack4(const char *p, int le);
+PyAPI_FUNC(double) PyFloat_Unpack8(const char *p, int le);
+PyAPI_FUNC(void) _PyFloat_InitState(void);
+
 #ifdef __cplusplus
 }
 #endif
