@@ -129,7 +129,7 @@ PyAPI_FUNC(int) PyContextVar_Get(struct _object *arg0, struct _object *arg1, str
 #define PyContextVar_New PyPyContextVar_New
 PyAPI_FUNC(struct _object *) PyContextVar_New(const char *arg0, struct _object *arg1);
 #define PyContextVar_Reset PyPyContextVar_Reset
-PyAPI_FUNC(struct _object *) PyContextVar_Reset(struct _object *arg0, struct _object *arg1);
+PyAPI_FUNC(int) PyContextVar_Reset(struct _object *arg0, struct _object *arg1);
 #define PyContextVar_Set PyPyContextVar_Set
 PyAPI_FUNC(struct _object *) PyContextVar_Set(struct _object *arg0, struct _object *arg1);
 #define PyCoro_Check PyPyCoro_Check
@@ -402,6 +402,8 @@ PyAPI_FUNC(int) PyGC_IsEnabled(void);
 PyAPI_FUNC(int) PyGILState_Check(void);
 #define PyGILState_Ensure PyPyGILState_Ensure
 PyAPI_FUNC(int) PyGILState_Ensure(void);
+#define PyGILState_GetThisThreadState PyPyGILState_GetThisThreadState
+PyAPI_FUNC(PyThreadState *) PyGILState_GetThisThreadState(void);
 #define PyGILState_Release PyPyGILState_Release
 PyAPI_FUNC(void) PyGILState_Release(int arg0);
 #define PyGen_Check PyPyGen_Check
@@ -483,9 +485,9 @@ PyAPI_FUNC(Signed) PyLong_AsLong(struct _object *arg0);
 #define PyLong_AsLongAndOverflow PyPyLong_AsLongAndOverflow
 PyAPI_FUNC(Signed) PyLong_AsLongAndOverflow(struct _object *arg0, int *arg1);
 #define PyLong_AsLongLong PyPyLong_AsLongLong
-PyAPI_FUNC(Signed) PyLong_AsLongLong(struct _object *arg0);
+PyAPI_FUNC(long long) PyLong_AsLongLong(PyObject * arg0);
 #define PyLong_AsLongLongAndOverflow PyPyLong_AsLongLongAndOverflow
-PyAPI_FUNC(Signed) PyLong_AsLongLongAndOverflow(struct _object *arg0, int *arg1);
+PyAPI_FUNC(long long) PyLong_AsLongLongAndOverflow(PyObject * arg0, int * arg1);
 #define PyLong_AsSize_t PyPyLong_AsSize_t
 PyAPI_FUNC(size_t) PyLong_AsSize_t(struct _object *arg0);
 #define PyLong_AsSsize_t PyPyLong_AsSsize_t
@@ -493,9 +495,9 @@ PyAPI_FUNC(Signed) PyLong_AsSsize_t(struct _object *arg0);
 #define PyLong_AsUnsignedLong PyPyLong_AsUnsignedLong
 PyAPI_FUNC(size_t) PyLong_AsUnsignedLong(struct _object *arg0);
 #define PyLong_AsUnsignedLongLong PyPyLong_AsUnsignedLongLong
-PyAPI_FUNC(size_t) PyLong_AsUnsignedLongLong(struct _object *arg0);
+PyAPI_FUNC(unsigned long long) PyLong_AsUnsignedLongLong(PyObject * arg0);
 #define PyLong_AsUnsignedLongLongMask PyPyLong_AsUnsignedLongLongMask
-PyAPI_FUNC(size_t) PyLong_AsUnsignedLongLongMask(struct _object *arg0);
+PyAPI_FUNC(unsigned long long) PyLong_AsUnsignedLongLongMask(PyObject * arg0);
 #define PyLong_AsUnsignedLongMask PyPyLong_AsUnsignedLongMask
 PyAPI_FUNC(size_t) PyLong_AsUnsignedLongMask(struct _object *arg0);
 #define PyLong_AsVoidPtr PyPyLong_AsVoidPtr
@@ -517,9 +519,9 @@ PyAPI_FUNC(struct _object *) PyLong_FromUnicode(wchar_t *arg0, Signed arg1, int
 #define PyLong_FromUnicodeObject PyPyLong_FromUnicodeObject
 PyAPI_FUNC(struct _object *) PyLong_FromUnicodeObject(struct _object *arg0, int arg1);
 #define PyLong_FromUnsignedLong PyPyLong_FromUnsignedLong
-PyAPI_FUNC(struct _object *) PyLong_FromUnsignedLong(size_t arg0);
+PyAPI_FUNC(PyObject *) PyLong_FromUnsignedLong(unsigned long arg0);
 #define PyLong_FromUnsignedLongLong PyPyLong_FromUnsignedLongLong
-PyAPI_FUNC(struct _object *) PyLong_FromUnsignedLongLong(size_t arg0);
+PyAPI_FUNC(PyObject *) PyLong_FromUnsignedLongLong(unsigned long long arg0);
 #define PyLong_FromVoidPtr PyPyLong_FromVoidPtr
 PyAPI_FUNC(struct _object *) PyLong_FromVoidPtr(void *arg0);
 #define PyMapping_Check PyPyMapping_Check
@@ -578,6 +580,10 @@ PyAPI_FUNC(int) PyModule_CheckExact(void * arg0);
 PyAPI_FUNC(struct _object *) PyModule_Create2(struct PyModuleDef *arg0, int arg1);
 #define PyModule_ExecDef PyPyModule_ExecDef
 PyAPI_FUNC(int) PyModule_ExecDef(struct _object *arg0, struct PyModuleDef *arg1);
+#define PyModule_FromDefAndSpec PyPyModule_FromDefAndSpec
+PyAPI_FUNC(PyObject *) PyModule_FromDefAndSpec(PyModuleDef * arg0, PyObject * arg1);
+#define PyModule_FromDefAndSpec2 PyPyModule_FromDefAndSpec2
+PyAPI_FUNC(PyObject *) PyModule_FromDefAndSpec2(PyModuleDef * arg0, PyObject * arg1, int arg2);
 #define PyModule_GetDict PyPyModule_GetDict
 PyAPI_FUNC(struct _object *) PyModule_GetDict(struct _object *arg0);
 #define PyModule_GetFilenameObject PyPyModule_GetFilenameObject
