@@ -1,17 +1,20 @@
 _ATTR_EXCEPTIONS = frozenset((
-    "__origin__",
     "__args__",
-    "__parameters__",
-    "__mro_entries__",
-    "__reduce_ex__",  # needed so we don't look up object.__reduce_ex__
-    "__reduce__",
+    "__class__",
     "__copy__",
     "__deepcopy__",
+    "__mro_entries__",
+    "__origin__",
+    "__parameters__",
+    "__reduce__",
+    "__reduce_ex__",  # needed so we don't look up object.__reduce_ex__
+    "__typing_unpacked_tuple_args__",
+    "__unpacked__",
 ))
 
 class GenericAlias:
 
-    __slots__ = ("__weakref__", "_origin", "_args", "_parameters", "_hash")
+    __slots__ = ("__weakref__", "_origin", "_args", "_parameters", "_hash", "__unpacked__")
 
     def __new__(cls, origin, args):
         result = super(GenericAlias, cls).__new__(cls)
@@ -19,7 +22,8 @@ class GenericAlias:
             args = (args, )
         result._origin = origin
         result._args = args
-        result._parameters = _make_parameters(args)
+        result._parameters = _collect_parameters(args)
+        result.__unpacked__ = False
         return result
 
     @property
@@ -54,12 +58,15 @@ class GenericAlias:
         inner = ', '.join([_repr_item(x) for x in self.__args__])
         if len(self.__args__) == 0:
             inner = "()"
-        return f"{_repr_item(self.__origin__)}[{inner}]"
+        star = '*' if self.__unpacked__ else ''
+        return f"{star}{_repr_item(self.__origin__)}[{inner}]"
 
     def __eq__(self, other):
         if not isinstance(other, GenericAlias):
             return NotImplemented
-        return self.__origin__ == other.__origin__ and self.__args__ == other.__args__
+        return (self.__origin__ == other.__origin__ and
+                self.__args__ == other.__args__ and
+                self.__unpacked__ == other.__unpacked__)
 
     def __getitem__(self, items):
         if not isinstance(items, tuple):
@@ -67,7 +74,10 @@ class GenericAlias:
         params = self.__parameters__
         args = self.__args__
         newargs = subs_parameters(self, args, params, items)
-        return GenericAlias(self.__origin__, tuple(newargs))
+        res = GenericAlias(self.__origin__, tuple(newargs))
+        if self.__unpacked__:
+            res.__unpacked__ = True
+        return res
 
     def __hash__(self):
         return hash(self.__origin__) ^ hash(self.__args__)
@@ -84,6 +94,9 @@ class GenericAlias:
         raise TypeError("isinstance() argument 2 cannot be a parameterized generic")
 
     def __reduce__(self):
+        if self.__unpacked__:
+            orig = GenericAlias(self.__origin__, self.__args__)
+            return (_make_starred, (orig, ))
         return (type(self), (self.__origin__, self.__args__))
 
     def __or__(self, other):
@@ -92,6 +105,21 @@ class GenericAlias:
     def __ror__(self, other):
         return _create_union(other, self)
 
+    def __iter__(self):
+        yield _make_starred(self)
+
+    @property
+    def __typing_unpacked_tuple_args__(self):
+        if self.__unpacked__ and self.__origin__ is tuple:
+            return self.__args__
+        return None
+
+
+def _make_starred(ga):
+    res = GenericAlias(ga.__origin__, ga.__args__)
+    res.__unpacked__ = True
+    return res
+
 
 def _repr_item(it):
     import typing
@@ -119,26 +147,38 @@ def _is_typevar(v):
     t = type(v)
     return t.__name__ == "TypeVar" and t.__module__ == "typing"
 
-def _make_parameters(args):
-    res = []
-    seen = set()
-    def add(x):
-        if x not in seen:
-            seen.add(x)
-            res.append(x)
-    for arg in args:
-        if _is_typevar(arg):
-            add(arg)
+def _collect_parameters(args):
+    """Collect all type variables and parameter specifications in args
+    in order of first appearance (lexicographic order).
+
+    For example::
+
+        >>> P = ParamSpec('P')
+        >>> T = TypeVar('T')
+        >>> _collect_parameters((T, Callable[P, T]))
+        (~T, ~P)
+    """
+    # copied from typing.py, for bootstrapping reasons
+    parameters = []
+    for t in args:
+        if isinstance(t, type):
+            # We don't want __parameters__ descriptor of a bare Python class.
+            pass
+        elif isinstance(t, tuple):
+            # `t` might be a tuple, when `ParamSpec` is substituted with
+            # `[T, int]`, or `[int, *Ts]`, etc.
+            for x in t:
+                for collected in _collect_parameters([x]):
+                    if collected not in parameters:
+                        parameters.append(collected)
+        elif hasattr(t, '__typing_subst__'):
+            if t not in parameters:
+                parameters.append(t)
         else:
-            try:
-                params = arg.__parameters__
-            except AttributeError:
-                pass
-            else:
-                if isinstance(params, tuple):
-                    for param in params:
-                        add(param)
-    return tuple(res)
+            for x in getattr(t, '__parameters__', ()):
+                if x not in parameters:
+                    parameters.append(x)
+    return tuple(parameters)
 
 def subs_tvars(obj, params, argitems):
     """If obj is a generic alias, substitute type variables params with
@@ -146,35 +186,55 @@ def subs_tvars(obj, params, argitems):
     and argitems is (str, int), return list[str]. If obj doesn't have a
     __parameters__ attribute or that's not a non-empty tuple, return a new
     reference to obj. """
+    from typing import TypeVarTuple
     subparams = getattr(obj, "__parameters__", ())
     if not subparams or not isinstance(subparams, tuple):
         return obj
     nparams = len(params)
     nsubparams = len(subparams)
     subargs = []
-    for arg in subparams:
+    for param in subparams:
         try:
-            arg = argitems[params.index(arg)]
+            arg = argitems[params.index(param)]
         except ValueError:
-            pass
-        subargs.append(arg)
+            arg = param
+        if isinstance(param, TypeVarTuple):
+            subargs.extend(arg)
+        else:
+            subargs.append(arg)
     return obj[tuple(subargs)]
 
 def subs_parameters(self, args, params, items):
+    from typing import _is_unpacked_typevartuple, _unpack_args
     nparams = len(params)
-    nitems = len(items)
     if nparams == 0:
-        raise TypeError("There are no type variables left in %r" % self)
+        raise TypeError(f"{self!r} is not a generic class")
+    items = _unpack_args(items)
+    for param in params:
+        prepare = getattr(param, '__typing_prepare_subst__', None)
+        if prepare is not None:
+            items = prepare(self, items)
+    nitems = len(items)
     if nparams != nitems:
-        raise TypeError("mismatched arguments for %r" % self)
-    args = args
+        direction = 'many' if nitems > nparams else 'few'
+        raise TypeError(f"Too {direction} arguments for {self}; actual {nitems}, expected {nparams}")
     newargs = []
-    for i, arg in enumerate(args):
-        if _is_typevar(arg):
-            iparam = params.index(arg)
-            newargs.append(items[iparam])
+    for i, old_arg in enumerate(args):
+        if isinstance(old_arg, type):
+            newargs.append(old_arg)
+            continue
+        unpack = _is_unpacked_typevartuple(old_arg)
+        meth = getattr(old_arg, '__typing_subst__', None)
+        if meth is not None:
+            iparam = params.index(old_arg)
+            arg = meth(items[iparam])
         else:
-            newargs.append(subs_tvars(arg, params, items))
+            arg = subs_tvars(old_arg, params, items)
+        if unpack:
+            newargs.extend(arg)
+        else:
+            newargs.append(arg)
+
     return newargs
 
 class UnionType:
@@ -201,7 +261,7 @@ class UnionType:
         for a in args:
             add_recurse(a)
         self._args = tuple(res)
-        self.__parameters__ = _make_parameters(args)
+        self.__parameters__ = _collect_parameters(args)
 
     @property
     def __args__(self):
