@@ -26,6 +26,7 @@ build_time_vars = {
     'CCSHARED': "-fPIC",
     'LDFLAGS': "-Wl,-Bsymbolic-functions",
     'LDSHARED': "cc -pthread -shared -Wl,-Bsymbolic-functions",
+    'LDCXXSHARED': "c++ -shared -Wl,-O1 -Wl,-Bsymbolic-functions",
     'EXT_SUFFIX': so_ext,
     'SHLIB_SUFFIX': ".so",
     'AR': "ar",
@@ -38,19 +39,19 @@ build_time_vars = {
     'SIZEOF_VOID_P': struct.calcsize("P"),
 }
 
-# LIBDIR should point to where the libpypy3.9-c.so file lives, on CPython
+# LIBDIR should point to where the libpypy3.11-c.so file lives, on CPython
 # it points to "mybase/lib". But that would require rethinking the PyPy
-# packaging process which copies pypy3 and libpypy3.9-c.so to the
+# packaging process which copies pypy3 and libpypy3.11-c.so to the
 # "mybase/bin" directory. Only when making a portable build (the default
 # for the linux buildbots) is there even a "mybase/lib" created, even so
 # the mybase/bin layout is left untouched.
 mybase = sys.base_prefix
 if sys.platform == 'win32':
-    build_time_vars['LDLIBRARY'] = 'libpypy3.9-c.dll'
+    build_time_vars['LDLIBRARY'] = 'libpypy3.11-c.dll'
     build_time_vars['INCLUDEPY'] = os.path.join(mybase, 'include')
     build_time_vars['LIBDIR'] = mybase
 else:
-    build_time_vars['LDLIBRARY'] = 'libpypy3.9-c.so'
+    build_time_vars['LDLIBRARY'] = 'libpypy3.11-c.so'
     build_time_vars['INCLUDEPY'] = os.path.join(mybase, 'include', 'pypy' + pydot)
     build_time_vars['LIBDIR'] = os.path.join(mybase, 'bin')
     build_time_vars['CONFINCLUDEPY'] = build_time_vars['INCLUDEPY']
@@ -85,8 +86,9 @@ if sys.platform[:6] == "darwin":
     arch = platform.machine()
     build_time_vars['CC'] += ' -arch %s' % (arch,)
     build_time_vars["LDFLAGS"] = "-undefined dynamic_lookup"
-    build_time_vars["LDSHARED"] = build_time_vars['CC'] + " -shared " + build_time_vars["LDFLAGS"]
-    build_time_vars['LDLIBRARY'] = 'libpypy3.9-c.dylib'
+    build_time_vars["LDSHARED"] = "clang -bundle -undefined dynamic_lookup "
+    build_time_vars["LDCXXSHARED"] = "clang++ -bundle -undefined dynamic_lookup "
+    build_time_vars['LDLIBRARY'] = 'libpypy3.11-c.dylib'
     # scikit-build checks this, it is left over from the NextStep rld linker
     build_time_vars['WITH_DYLD'] = 1
     if "CXX" in build_time_vars:
@@ -97,5 +99,8 @@ if sys.platform[:6] == "darwin":
     #
     # In any case, keep this in sync with DARWIN_VERSION_MIN in
     # rpython/translator/platform/darwin.py and Lib/_osx_support.py
-    build_time_vars['MACOSX_DEPLOYMENT_TARGET'] = '10.13'
+    if arch == "arm64":
+        build_time_vars['MACOSX_DEPLOYMENT_TARGET'] = '11.0'
+    else:
+        build_time_vars['MACOSX_DEPLOYMENT_TARGET'] = '10.15'
 
