@@ -44,9 +44,9 @@ def Tcl_AppInit(app):
         tcl_path = tcl_path.replace(sep, '/')
         tk_path = tk_path.replace(sep, '/')
     else:
-        lib_path = join(dirname(dirname(dirname(__file__))), 'lib')
-        tcl_path = join(lib_path, 'tcl')
-        tk_path = join(lib_path, 'tk')
+        lib_path = join(dirname(dirname(dirname(__file__))))
+        tcl_path = join(lib_path, 'tcl8.6')
+        tk_path = join(lib_path, 'tk8.6')
     if exists(tcl_path):
         tklib.Tcl_Eval(app.interp, 'set tcl_library "{0}"'.format(tcl_path).encode('utf-8'))
     if exists(tk_path):    
@@ -510,12 +510,11 @@ class TkApp(object):
     def getboolean(self, s):
         if isinstance(s, int):
             return bool(s)
-        try:
-            s = s.encode('utf-8')
-        except AttributeError:
-            raise TypeError
-        if b'\x00' in s:
+        if isinstance(s, Tcl_Obj):
+            s = s.string
+        if '\x00' in s:
             raise TypeError
+        s = s.encode('utf-8')
         v = tkffi.new("int*")
         res = tklib.Tcl_GetBoolean(self.interp, s, v)
         if res == tklib.TCL_ERROR:
@@ -525,12 +524,11 @@ class TkApp(object):
     def getint(self, s):
         if isinstance(s, int):
             return s
-        try:
-            s = s.encode('utf-8')
-        except AttributeError:
-            raise TypeError
-        if b'\x00' in s:
+        if isinstance(s, Tcl_Obj):
+            s = s.string
+        if '\x00' in s:
             raise TypeError
+        s = s.encode('utf-8')
         if tklib.HAVE_LIBTOMMATH or tklib.HAVE_WIDE_INT_TYPE:
             value = tklib.Tcl_NewStringObj(s, -1)
             if not value:
@@ -552,12 +550,11 @@ class TkApp(object):
     def getdouble(self, s):
         if isinstance(s, (float, int)):
             return float(s)
-        try:
-            s = s.encode('utf-8')
-        except AttributeError:
-            raise TypeError
-        if b'\x00' in s:
+        if isinstance(s, Tcl_Obj):
+            s = s.string
+        if '\x00' in s:
             raise TypeError
+        s = s.encode('utf-8')
         v = tkffi.new("double*")
         res = tklib.Tcl_GetDouble(self.interp, s, v)
         if res == tklib.TCL_ERROR:
